// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PixelPerfectCamera.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PixelPerfectCamera::PixelPerfectCamera(SceneItem *i) : SceneItemComponent(i) {

}

std::vector<SceneItemComponent*> PixelPerfectCamera::getDependencies() {
  return std::vector<SceneItemComponent*>{
    (this->camera = this->item->getComponent<Camera>())
  };
}

void PixelPerfectCamera::onRenderTargetResized(
  RenderTarget *t, float_t w, float_t h
) {
  this->updateDimensions();
}

void PixelPerfectCamera::updateDimensions() {
  float_t w, h;
  assertNotNull(this->camera);

  auto target = this->camera->getRenderTarget();
  switch(this->camera->type) {
    case CAMERA_TYPE_ORTHONOGRAPHIC:
      w = target->getWidth() / 2.0f / this->scale;
      h = target->getHeight() / 2.0f / this->scale;
      camera->orthoLeft = -w;
      camera->orthoRight = w;
      camera->orthoTop = h;
      camera->orthoBottom = -h;
      camera->updateProjection();
      break;
    
    case CAMERA_TYPE_PERSPECTIVE:
      assertDeprecated();
      break;
    
    default:
      assertUnreachable();
  }
}

void PixelPerfectCamera::onStart() {
  assertNotNull(this->camera);
  this->updateDimensions();
  camera->getRenderTarget()->eventRenderTargetResized
    .addListener(this, &PixelPerfectCamera::onRenderTargetResized)
  ;
}

PixelPerfectCamera::~PixelPerfectCamera() {
  camera->getRenderTarget()->eventRenderTargetResized
    .removeListener(this, &PixelPerfectCamera::onRenderTargetResized)
  ;
}